# -*- coding: utf-8 -*-
"""This module contains the controller classes of the application."""

# symbols which are imported by "from hoyataxa.controllers import *"
__all__ = ['Root']

# standard library imports
# import logging
import datetime

# third-party imports
from turbogears import controllers, expose, flash

# project specific imports
# from hoyataxa import model
# from hoyataxa import json


# log = logging.getLogger("hoyataxa.controllers")

from model import Taxonomy, Name

from turbogears import validate, validators
from turbogears import widgets, error_handler

class SearchFields(widgets.WidgetsList):
    query = widgets.TextField(label="Search Term")
    mode = widgets.SingleSelectField(label="Search Mode",
                                     options=["Starts with",
                                              "Ends with",
                                              "Contains"],
                                     default="Contains")

class SearchFieldsSchema(validators.Schema):
    query = validators.All(validators.PlainText(),
                           validators.String(min=3,strip=True))
    mode = validators.OneOf(["Starts with","Ends with","Contains"])

search_form = widgets.TableForm(
    fields = SearchFields(),
    validator = SearchFieldsSchema(),
    action = "search",
    submit_text = "Search"
    )

class Root(controllers.RootController):
    """The root controller of the application."""

    @expose(template="hoyataxa.templates.welcome")
    def index(self):
        """"Show the welcome page."""
        return dict(form=search_form,
                    title="All your taxa are belong to us!!!!!")

    @expose(template="hoyataxa.templates.search")
    @expose(template="hoyataxa.templates.searchxml",
            as_format="xml", format="xml")
    @validate(form=search_form)
    @error_handler(index)
    def search(self,query,mode):
        if mode == 'Starts with':
            names = Name.select(Name.q.name.startswith(query))
        elif mode == 'Ends with':
            names = Name.select(Name.q.name.endswith(query))
        elif mode == 'Contains':
            names = Name.select(Name.q.name.contains(query))
        return dict(names=names,query=query,mode=mode)

    @expose(template="hoyataxa.templates.taxa")
    @expose(template="hoyataxa.templates.taxaxml",
            as_format="xml", format="xml")
    def taxa(self,taxid):
        """"Show the welcome page."""
        t = Taxonomy.byTaxid(int(taxid))
        title = "Taxonomy: "+taxid
        lineage = []
        t1 = t
        while t1.taxid != 1:
            if t1.rank != 'no rank':
                lineage.append(t1)
            t1 = t1.parent
        lineage.reverse()
        return dict(title=t.scientific_name,
                    taxa=t, lineage=lineage)
